<?php

use Illuminate\Database\Capsule\Manager as Capsule;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;

class CreateUserPhonesTable extends Migration {

  /**
   * Run the migrations.
   *
   * @return void
   */
  public function up()
  {
    Capsule::schema()->create($this->getTableName(), function(Blueprint $table)
    {
      $table->increments('id');
      $table->unsignedInteger('user_id');
      $table->string('phone', 36);
      $table->string('type', 20);
      $table->boolean('primary')->default(false);
      $table->boolean('verified')->default(false);
      $table->softDeletes();
      $table->timestamps();

      $table->foreign('user_id')->references('id')->on($this->getUserTableName());
    });
  }

  /**
   * Reverse the migrations.
   *
   * @return void
   */
  public function down()
  {
    Capsule::schema()->dropIfExists($this->getTableName());
  }
  
  /**
   * Get the table name from the config file
   *
   * @return string the table name
   */
  private function getTableName() {
    return config('authentication.table_names.user_phones', 'user_phones');
  }

  /**
   * Get the Users table name from the config file
   *
   * @return string The name of the Users table
   */
  private function getUserTableName() {
    return config('authentication.table_names.users', 'users');
  }

}
