<?php

namespace spec\Myriad\Authentication\Lib\Jobs;

use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Illuminate\Contracts\Validation\Factory;
use Illuminate\Contracts\Validation\Validator;
use Myriad\Authentication\Lib\Repositories\IInstallationRepository;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Authentication\Models\User;
use Myriad\Installations\Models\Installation;
use Myriad\Authentication\Models\AccessToken;
use Myriad\Installations\InstallationManager;
use Illuminate\Contracts\Auth\UserProvider;

class CreateUserInstallationAccessTokenSpec extends ObjectBehavior
{
  function let()
  {
    $this->beConstructedWith([]);
  }

  function it_is_initializable()
  {
    $this->shouldHaveType('Myriad\Authentication\Lib\Jobs\CreateUserInstallationAccessToken');
  }

  function it_should_create_an_access_token(
    Factory $factory,
    Validator $validator,
    UserProvider $userProvider,
    User $user,
    InstallationManager $installationManager,
    Installation $installation,
    IAccessTokenRepository $accessTokenRepository,
    AccessToken $accessToken
  ) {
    $req = [
      'user_id' => '1',
      'installation_uuid' => '2'
    ];

    $factory->make(Argument::any(), Argument::type('array'))
      ->willReturn($validator)->shouldBeCalled();
    $validator->fails()->willReturn(false)->shouldBeCalled();

    $this->beConstructedWith($req);

    $userProvider->retrieveById($req['user_id'])
      ->willReturn($user);
    $installationManager->findInstallationByUUID($req['installation_uuid'])
      ->willReturn($installation);

    $accessTokenRepository->createAccessToken($user, $installation)
      ->willReturn($accessToken);

    $this->handle($factory, $userProvider,
      $installationManager, $accessTokenRepository)
      ->shouldReturn($accessToken);
  }
}
