<?php

namespace Myriad\Authentication\Http\Controllers;

use Illuminate\Routing\Controller;
use Myriad\Authentication\Exceptions\InvalidCodeException;
use Myriad\Authentication\Lib\Repositories\IAccessTokenRepository;
use Myriad\Installations\InstallationManager;
use Myriad\Authentication\Lib\Services\IEmailConfirmer;
use Illuminate\Contracts\Auth\Guard;
use Myriad\Authentication\Lib\Services\IPhoneNumberConfirmer;
use Myriad\Authentication\Http\Requests\SendVerificationCodeRequest;
use Myriad\Authentication\Http\Requests\VerifyVerificationCodeRequest;
use Illuminate\Http\Response;
use Myriad\Authentication\Lib\Repositories\IVerificationCodeRepository;

class VerificationCodeController extends Controller {

  public function send(
    SendVerificationCodeRequest $request,
    Guard $guard,
    IEmailConfirmer $emailConfirmer,
    IPhoneNumberConfirmer $phoneNumberConfirmer
  ) {
    $attributes['primary'] = !$guard->check();

    //TODO: move these to jobs
    if ($request->has('email')) {
      $emailConfirmer->sendConfirmationEmail(
        $request->email,
        $guard->user(),
        $attributes
      );
    } else {
      $phoneDigits = $this->keepOnlyDigits($request->phone);
      $phoneNumberConfirmer->sendConfirmationTextMessage(
        $phoneDigits,
        $guard->user(),
        $attributes
      );
    }

    return new Response('', 204);
  }

  private function keepOnlyDigits($phone) {
    $matches = [];
    preg_match_all('/[0-9]/', $phone, $matches);

    if (count($matches) === 0) {
      return '';
    }

    return implode($matches[0]);
  }

  public function verify(
    VerifyVerificationCodeRequest $request,
    IAccessTokenRepository $accessTokenRepository,
    InstallationManager $manager,
    IVerificationCodeRepository $verificationCodeRepository,
    Guard $guard
  ) {
    $codes = $verificationCodeRepository->findVerificationCodes($request->contact);

    $matchingCode = null;
    foreach ($codes as $code) {
      if ($code->attemptToMatch($request->code)) {
        $matchingCode = $code;
        break;
      }
    }

    if (!$matchingCode) {
      throw new InvalidCodeException("No valid code found matching " . $request->code);
    }

    $user = $matchingCode->performVerificationStrategy($request->code);
    $guard->setUser($user);

    $installation = $manager->getInstallation();
    $accessToken = $accessTokenRepository->createAccessToken($user, $installation);

    return [
      'data' => [
        'access_token' => $accessToken->token,
        'expires_at' => $accessToken->expires_at->toIso8601String()
      ]
    ];
  }

}
