<?php

namespace Myriad\Authentication\Models;

use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Database\Eloquent\Model;
use Myriad\Installations\Models\Installation;
use Carbon\Carbon;
use Illuminate\Contracts\Auth\Authenticatable;

class AccessToken extends Model
{
  use SoftDeletes;

  /**
   * The attributes that are mass assignable.
   *
   * @var array
   */
  protected $fillable = [
    'token',
    'owner_type',
    'owner_id',
    'client_type',
    'client_id',
    'expires_at',
  ];

  /**
   * The attributes that should be hidden for arrays.
   *
   * @var array
   */
  protected $hidden = [
    'token',
  ];

  /**
   * The attributes that should be mutated to dates.
   *
   * @var array
   */
  protected $dates = [ 'expires_at' ];

  /**
   * Get the owner object for this token.
   *
   * @return \Illuminate\Database\Eloquent\Relations\MorphTo
   */
  public function owner()
  {
    return $this->morphTo();
  }

  /**
   * Get the client object for this token.
   *
   * @return \Illuminate\Database\Eloquent\Relations\MorphTo
   */
  public function client()
  {
    return $this->morphTo();
  }

  /**
   * Is this token expired
   *
   */
  public function isExpired() {
    return Carbon::now()->lt($this->expires_at);
  }

  public function isForUser(Authenticatable $user) {
    return $this->owner->getAuthIdentifier() === $user->getAuthIdentifier();
  }
}
